from _Framework.ButtonElement import ButtonElement

from fingerboardcodes import *
from livezonelayout import channelAndNoteForLed
import diagnostic

MIDI_NOTE_ON_STATUS = 144
MIDI_NOTE_OFF_STATUS = 128

class FingerboardButton(ButtonElement):
	"""Represents a ButtonElement adapted for the interface to the Ztar fingerboard."""
	def __init__(self, isMomentary, messageType, led):
		self._ledNumber = led
		self._channelOut, self._note = channelAndNoteForLed(led)
		ButtonElement.__init__(self, isMomentary, messageType, self._channelOut, self._note)
		self._lastSentFunction = -1
	
	def disconnect(self):
		ButtonElement.disconnect(self)
	
	def send_value(self, value, force_send=False):
		"""When this button is assigned to a component using a set_XXX_button() method, this method is registered
as a value listener with that component, and is called when the component calls its update() method. The values
it sends are customized using the set_XXX_value() methods on the component."""
		assert (value != None)
		assert isinstance(value, int)
		assert (value in range(128))
		if (force_send or ((value != self._last_sent_value) and self._is_being_forwarded)):
			statusByte = MIDI_NOTE_ON_STATUS if value != 0 else MIDI_NOTE_OFF_STATUS
			statusByte += self._channelOut
			if self.send_midi((statusByte, self._note, value)):
				self._last_sent_value = value
				if self._report_output:
					is_input = True
					self._report_value(value, (not is_input))

	def resendLastValue(self):
		if self._last_sent_value in range(128):
			statusByte = MIDI_NOTE_ON_STATUS if self._last_sent_value != 0 else MIDI_NOTE_OFF_STATUS
			statusByte += self._channelOut
			if self.send_midi((statusByte, self._note, self._last_sent_value)):
				diagnostic.logMessage('Sent %d to LED %d' % (self._last_sent_value, self._ledNumber))
				if self._report_output:
					self._report_value(self._last_sent_value, False)
	
	def send_function(self, functionCode, force_send=False):
		"""Uses the designated function channel (in fingerboardcodes.py) to send custom functions to the Ztar. This
is used when a button normally assigned to an LED needs to send the Ztar a function code, which may be different
from the channel it normally communicates on."""
		assert (functionCode != None)
		if (force_send or ((functionCode != self._lastSentFunction) and self._is_being_forwarded)):
			statusByte = MIDI_NOTE_ON_STATUS + FUNCTION_CHANNEL
			functionCode = functionCodeForLed(functionCode, self._ledNumber)
			
			# When sending functions to the fingerboard, the velocity value identifies the LED's offset from 128
			velocityValue = self._ledNumber % 128
			if self.send_midi((statusByte, functionCode, velocityValue)):
				self._lastSentFunction = functionCode
