from _Framework.CompoundComponent import CompoundComponent
from _Framework.SceneComponent import SceneComponent

import diagnostic
from LiveZoneClipSlot import LiveZoneClipSlot
from fingerboardcodes import FUNC_BRIGHT, FUNC_BLINKON, FUNC_BLINKOFF

class LiveZoneScene(SceneComponent):
	"""Represents a scene controlled through the LiveZone on the Ztar fingerboard."""
	def __init__(self, num_slots, tracks_to_use_callback):
		assert (num_slots != None)
		assert isinstance(num_slots, int)
		assert (num_slots >= 0)
		assert tracks_to_use_callback != None
		assert dir(tracks_to_use_callback).count('im_func') is 1	# Ensure the callback came from Live
		CompoundComponent.__init__(self)
		self._scene = None
		self._clip_slots = []
		self._tracks_to_use_callback = tracks_to_use_callback
		for i in range(num_slots):
			newSlot = self._create_clip_slot()	# Creates LiveZoneClipSlot
			self._clip_slots.append(newSlot)
			self.register_components(newSlot)
		self._launch_button = None
		self._default_value = 0
		self._triggered_value = 127
		self._track_offset = 0
		return None
	
	def setClipLaunchButtons(self, buttons):
		if len(buttons) < len(self._clip_slots):
			diagnostic.logMessage('Scene: Too few launch buttons to assign to clip slots.')
			return
		
		for i, clipSlot in enumerate(self._clip_slots):
			if clipSlot._launch_button:
				clipSlot._launch_button.turn_off()
			clipSlot.set_launch_button(buttons[i])
	
	def set_default_value(self, value):
		if value not in range(128): return None
		self._default_value = value
	
	def set_triggered_value(self, value):
		if value not in range(128): return None
		self._triggered_value = value
	
	def _on_is_triggered_changed(self):
		"""Responds to the Live event generated by the connected scene when its triggered state changes. Also called
by the update() method when the scene is connected and enabled."""
		if not self._scene: return None
		
		if self.is_enabled() and self._launch_button:
			if self._scene.is_triggered:
				self._launch_button.send_value(self._triggered_value)
				self._launch_button.send_function(FUNC_BLINKON)
				self._launch_button.send_function(FUNC_BRIGHT)
			else:
				self._launch_button.send_value(self._default_value)
				self._launch_button.send_function(FUNC_BLINKOFF)
				self._launch_button.send_function(FUNC_BRIGHT)
	
	def _create_clip_slot(self):
		return LiveZoneClipSlot()
